<?php
class fxjson{
	const API_URL_PREFIX = "http://m.xiami.com/web/get-songs?type=0&rtype=";
	const SONG_URL = "song&id=";
	const ALBUM_URL = "album&id=";
	const COLLECT_URL = "collect&id=";
	const TOKEN_KEY = "&_xiamitoken=";
	const SONG_KEY_PREFIX = "xiami/song/";
	const ALBUM_KEY_PREFIX = "xiami/album/";
	const COLLECT_KEY_PREFIX = "xiami/collect/";
	const XIAMI_TOKEN_KEY = "_xiamitoken";

	private $token;

	public function __construct(){
		$this->get_token();
	}

	public function song($song_id){
		$key = self::SONG_KEY_PREFIX . $song_id;
		$url = self::API_URL_PREFIX . self::SONG_URL . $song_id . self::TOKEN_KEY . $this->token;

		$cache = $this->get_cache($key);
		if( $cache ) return $cache;

		$response = $this->http($url);

		if(  $response && $response['status'] == "ok" ){
		    $result = array(
			    "song_id" => $response["data"][0]["id"],
			    "song_title" => $response["data"][0]["title"],
				"song_author" => $response["data"][0]["author"],
				"song_cover" => $response["data"][0]["cover"],
				"song_src" => $response["data"][0]["src"]
			);

		    $this->set_cache($key, $result);

		    return $result;
		}

		return false;
	}
	public function netease_song($music_id)
	{
		$key = "/netease/song/$music_id";

		$cache = $this->get_cache($key);
		if( $cache ) return $cache;

		$url = "http://music.163.com/api/song/detail/?id=" . $music_id . "&ids=%5B" . $music_id . "%5D";
    	$response = $this->netease_http($url);
		
		if( $response["code"]==200 && $response["songs"] ){
		//print_r($response["songs"]);
			//处理音乐信息
			$mp3_url = $response["songs"][0]["mp3Url"];
			$mp3_url = str_replace("http://m", "http://p", $mp3_url);
			$music_name = $response["songs"][0]["name"];
			$mp3_cover = $response["songs"][0]["album"]["picUrl"];
			$artists = array();
	
			foreach ($response["songs"][0]["artists"] as $artist) {
			    $artists[] = $artist["name"];
			}

			$artists = implode(",", $artists);

		    $result = array(
			    "song_id" => $music_id,
			    "song_title" => $music_name,
				"song_author" => $artists,
				"song_src" => $mp3_url,
				"song_cover" => $mp3_cover
			);

		    $this->set_cache($key, $result);

		    return $result;
		}

		return false;	
	}
	
	private function netease_http($url)
	{
	    $refer = "http://music.163.com/";
	    $header[] = "Cookie: " . "appver=1.5.0.75771;";
	    $ch = curl_init();
	    curl_setopt($ch, CURLOPT_URL, $url);
	    curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
	    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	    curl_setopt($ch, CURLOPT_BINARYTRANSFER, true);
	    curl_setopt($ch, CURLOPT_REFERER, $refer);
	    $cexecute = curl_exec($ch);
	    curl_close($ch);

		if ($cexecute) {
			$result = json_decode($cexecute, true);
			return $result;
		}else{
			return false;
		}
	}
	private function http($url, $json=true){
		if( !$url ){
			return false;
		}

        $header = array(
            'Host: m.xiami.com',
            'User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 7_1_2 like Mac OS X) AppleWebKit/537.51.2 (KHTML, like Gecko) Version/7.0 Mobile/11D257 Safari/9537.53',
            'Cookie: _xiamitoken='.$this->token.'; visit=1',
            'Proxy-Connection:keep-alive',
            'X-Requested-With:XMLHttpRequest',
        	'X-FORWARDED-FOR:42.156.140.238', 
			'CLIENT-IP:42.156.140.238'
        );

		$ch = curl_init($url);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_REFERER, 'http://m.xiami.com/');
        $cexecute = curl_exec($ch);
		@curl_close($ch);

		if ($cexecute) {
			if($json) $result = json_decode($cexecute, true);
			return $result;
		}else{
			return false;
		}
	}
	private function get_token(){
		$token = get_transient(self::XIAMI_TOKEN_KEY);

		if( $token ){
			$this->token = $token;
		}else{
			$XM_head = wp_remote_head('http://m.xiami.com', array(
				'headers' => array(
				    'Host: m.xiami.com',
				    'User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 7_1_2 like Mac OS X) AppleWebKit/537.51.2 (KHTML, like Gecko) Version/7.0 Mobile/11D257 Safari/9537.53',
				    'Proxy-Connection:keep-alive',
				    'X-Requested-With:XMLHttpRequest',
        			'X-FORWARDED-FOR:42.156.140.238', 
					'CLIENT-IP:42.156.140.238'
				)
			));

			$cookies = $XM_head['cookies'];

			foreach ($cookies as $key => $cookie) {
				if( $cookie->name == '_xiamitoken' ){
					$this->token = $cookie->value;

					set_transient(self::XIAMI_TOKEN_KEY, $this->token, 60*60*100);
				}
			}			
		}
	}
	public function get_cache($key){
		$cache = get_transient($key);
		return $cache === false ? false : json_decode($cache);
	}

	public function set_cache($key, $value){
		$value  = json_encode($value);
		set_transient($key, $value, 60*60*2);
	}

	public function clear_cache($key){
		delete_transient($key);
	}
}